#![windows_subsystem = "windows"]
use bevy::{prelude::*, winit::WinitSettings};
use bevy_egui::{
    egui::{self},
    EguiContexts, EguiPlugin,
};

#[cfg(windows)]
const LINE_ENDING: &str = "\r\n";
#[cfg(not(windows))]
const LINE_ENDING: &str = "\n";

#[derive(Resource, Debug)]
struct InputText {
    value: String,
}

#[derive(Resource, Debug)]
struct NeedsSort {
    value: bool,
}

#[derive(Resource, Debug)]
struct NeedsStrip {
    value: bool,
}

fn main() {
    App::new()
        .insert_resource(WinitSettings::desktop_app())
        .insert_resource(InputText {
            value: "".to_string(),
        })
        .insert_resource(NeedsSort { value: false })
        .insert_resource(NeedsStrip { value: false })
        .add_plugins(DefaultPlugins.set(WindowPlugin {
            primary_window: Some(Window {
                title: "dsort".to_string(),
                ..Default::default()
            }),
            ..Default::default()
        }))
        .add_plugins(EguiPlugin)
        .add_systems(Update, ui_system)
        .add_systems(Update, process_text)
        .run();
}

fn process_text(
    mut needs_sort: ResMut<NeedsSort>,
    mut needs_strip: ResMut<NeedsStrip>,
    mut input_text: ResMut<InputText>,
) {
    let needs_processing = needs_sort.value || needs_strip.value;

    if needs_processing {
        let mut arr = input_text.value.lines().collect::<Vec<_>>();

        if needs_strip.value {
            arr.retain(|&x| !x.is_empty());
            needs_strip.value = false;
        }

        if needs_sort.value {
            arr.sort_by(|&a, &b| vsort::compare(a, b));
            needs_sort.value = false;
        }

        input_text.value = arr.join(LINE_ENDING);
    }
}

fn button(label: &str) -> egui::Button {
    egui::Button::new(label).sense(egui::Sense::click_and_drag())
}

fn button_clicked(button: egui::Response) -> bool {
    button.clicked() || button.drag_stopped()
}

fn ui_system(
    mut contexts: EguiContexts,
    mut input_text: ResMut<InputText>,
    mut needs_sort: ResMut<NeedsSort>,
    mut needs_strip: ResMut<NeedsStrip>,
) {
    contexts.ctx_mut().set_visuals(egui::Visuals::light());

    egui::CentralPanel::default().show(contexts.ctx_mut(), |ui| {
        let window_size = ui.available_size();
        let height = window_size[1];

        ui.with_layout(egui::Layout::top_down(egui::Align::Center), |ui| {
            ui.with_layout(egui::Layout::left_to_right(egui::Align::LEFT), |ui| {
                let sort_button = ui.add(button("Sort"));
                if button_clicked(sort_button) {
                    needs_sort.value = true;
                }

                let strip_button = ui.add(button("Remove Blanks"));
                if button_clicked(strip_button) {
                    needs_strip.value = true;
                }

                let copy_button = ui.add(button("Copy"));
                if button_clicked(copy_button) {
                    ui.output_mut(|o| o.copied_text = input_text.value.clone());
                }

                let clear_button = ui.add(button("Clear"));
                if button_clicked(clear_button) {
                    input_text.value = "".to_string();
                }
            });

            ui.add_space(5.);

            egui::ScrollArea::vertical().show(ui, |ui| {
                ui.add_sized(
                    [ui.available_width(), height],
                    egui::TextEdit::multiline(&mut input_text.value),
                );
            });
        });
    });
}
